//----------------------------------------------------
// RealTimeCellShading.cs (c) 2007 by Charles Petzold
//----------------------------------------------------
using System;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Media.Media3D;
using Petzold.Media3D;

namespace Petzold.RealTimeCellShading
{
    public partial class RealTimeCellShading : Window
    {
        Viewport3DVisual vis;
        RenderTargetBitmap bitmap;
        AxisAngleRotation3D rotateCameraHorz, rotateCameraVert, 
                            rotateTeapotHorz, rotateTeapotVert;

        [STAThread]
        public static void Main()
        {
            Application app = new Application();
            app.Run(new RealTimeCellShading());
        }

        public RealTimeCellShading()
        {
            InitializeComponent();

            // Create Viewport3DVisual
            vis = new Viewport3DVisual();

            // Create ModelVisual3D for teapot.
            ModelVisual3D modvis = new ModelVisual3D();
            GeometryModel3D geomod = new GeometryModel3D();
            TeapotMesh mesh = new TeapotMesh();
            geomod.Geometry = mesh.Geometry;
            geomod.Material = new DiffuseMaterial(Brushes.Lime);
            geomod.BackMaterial = new DiffuseMaterial(Brushes.Black);
            modvis.Content = geomod;
            Transform3DGroup xformgrp = new Transform3DGroup();
            rotateTeapotHorz = new AxisAngleRotation3D(new Vector3D(0, 1, 0), 0);
            xformgrp.Children.Add(new RotateTransform3D(rotateTeapotHorz));
            rotateTeapotVert = new AxisAngleRotation3D(new Vector3D(1, 0, 0), 0);
            xformgrp.Children.Add(new RotateTransform3D(rotateTeapotVert));
            modvis.Transform = xformgrp;
            vis.Children.Add(modvis);

            // Create ModelVisual3D for lighting.
            modvis = new ModelVisual3D();
            Model3DGroup modgrp = new Model3DGroup();
            modgrp.Children.Add(new AmbientLight(Color.FromRgb(64, 64, 64)));
            modgrp.Children.Add(new DirectionalLight(Color.FromRgb(192, 192, 192),
                                                     new Vector3D(2, -3, -1)));
            modvis.Content = modgrp;
            vis.Children.Add(modvis);

            // Create camera.
            PerspectiveCamera cam = new PerspectiveCamera();
            cam.Position = new Point3D(0, 0, 5);
            xformgrp = new Transform3DGroup();
            rotateCameraHorz = new AxisAngleRotation3D(new Vector3D(0, 1, 0), 0);
            xformgrp.Children.Add(new RotateTransform3D(rotateCameraHorz));
            rotateCameraVert = new AxisAngleRotation3D(new Vector3D(1, 0, 0), 0);
            xformgrp.Children.Add(new RotateTransform3D(rotateCameraVert));
            cam.Transform = xformgrp;
            vis.Camera = cam;
        }

        void BorderOnSizeChanged(object sender, SizeChangedEventArgs args)
        {
            vis.Viewport = new Rect(args.NewSize);
            bitmap = new RenderTargetBitmap((int)args.NewSize.Width,
                                            (int)args.NewSize.Height,
                                            96, 96, PixelFormats.Default);
            UpdateBitmap();
        }

        void HorizontalScrollOnValueChanged(object sender,
                                            RoutedPropertyChangedEventArgs<double> args)
        {
            rotateCameraHorz.Angle = args.NewValue;
            UpdateBitmap();
        }

        void VerticalScrollOnValueChanged(object sender,
                                          RoutedPropertyChangedEventArgs<double> args)
        {
            rotateCameraVert.Angle = args.NewValue;
            UpdateBitmap();
        }

        void HorizontalSliderOnValueChanged(object sender,
                                            RoutedPropertyChangedEventArgs<double> args)
        {
            rotateTeapotHorz.Angle = args.NewValue;
            UpdateBitmap();
        }

        void VerticalSliderOnValueChanged(object sender,
                                          RoutedPropertyChangedEventArgs<double> args)
        {
            rotateTeapotVert.Angle = args.NewValue;
            UpdateBitmap();
        }

        void UpdateBitmap()
        {
            bitmap.Clear();
            bitmap.Render(vis);

            // Alter bitmap bits.
            uint[] bits = new uint[bitmap.PixelWidth * bitmap.PixelHeight];
            bitmap.CopyPixels(bits, 4 * bitmap.PixelWidth, 0);

            for (int i = 0; i < bitmap.PixelWidth * bitmap.PixelHeight; i++)
                bits[i] = bits[i] & 0xFFC0C0C0;

            BitmapSource bitmap2 = BitmapSource.Create(bitmap.PixelWidth, bitmap.PixelHeight,
                                                       bitmap.DpiX, bitmap.DpiY,
                                                       bitmap.Format, null, bits,
                                                       4 * bitmap.PixelWidth);
            brush.ImageSource = bitmap2;
        }
    }
}